@@ -6,6 +6,8 @@ from django.db import connections
 from django.db.models import Model
 
 from .. import smartsql
+from ..smartsql.compilers import mysql
+from ..smartsql.compilers import sqlite
 from .signals import field_conversion
 
 try:
@@ -17,13 +19,12 @@ except NameError:
     integer_types = (int,)
 
 SMARTSQL_ALIAS = getattr(settings, 'SQLBUILDER_SMARTSQL_ALIAS', 's')
-SMARTSQL_DIALECTS = {
-    'sqlite3': 'sqlite',
-    'mysql': 'mysql',
-    'postgresql': 'postgres',
-    'postgresql_psycopg2': 'postgres',
-    'postgis': 'postgres',
-    'oracle': 'oracle',
+SMARTSQL_COMPILERS = {
+    'sqlite3': sqlite.compile,
+    'mysql': mysql.compile,
+    'postgresql': smartsql.compile,
+    'postgresql_psycopg2': smartsql.compile,
+    'postgis': smartsql.compile,
 }
 
 cr = copy.copy(smartsql.cr)
@@ -38,142 +39,105 @@ class classproperty(object):
         return self.getter(owner)
 
 
-@cr('QuerySet')
-class QS(smartsql.QS):
-    """Query Set adapted for Django."""
+class Result(smartsql.Result):
 
     _cache = None
     _using = 'default'
-    model = None
+    _model = None
 
-    def __init__(self, tables=None):
-        super(QS, self).__init__(tables=tables)
-        if isinstance(tables, (Table, TableAlias)):
-            self.model = tables.model
-            self._using = self.model.objects.db
-
-    def clone(self):
-        self = super(QS, self).clone()
-        self._cache = None
-        return self
-
-    def fill_cache(self):
-        if self._cache is None:
-            self._cache = list(self.iterator())
-        return self
+    def __init__(self, model):
+        self._model = model
+        self._using = self._model.objects.db
+        self.set_compiler()
 
     def __len__(self):
-        """Returns length or list."""
         self.fill_cache()
         return len(self._cache)
 
-    def count(self):
-        """Returns length or list."""
-        if self._cache is not None:
-            return len(self._cache)
-        return super(QS, self).count()
-
-    def iterator(self):
-        return self.execute()
-
     def __iter__(self):
-        """Returns iterator."""
         self.fill_cache()
         return iter(self._cache)
 
     def __getitem__(self, key):
-        """Returns sliced self or item."""
         if self._cache:
             return self._cache[key]
         if isinstance(key, integer_types):
-            self = super(QS, self).__getitem__(key)
+            self._query = super(Result, self).__getitem__(key)
             return list(self)[0]
-        return super(QS, self).__getitem__(key)
+        return super(Result, self).__getitem__(key)
+
+    def execute(self):
+        cursor = connections[self._using].cursor()
+        cursor.execute(*self.compile(self._query))
+        return cursor
+
+    insert = update = delete = execute
+
+    def select(self):
+        return self
+
+    def count(self):
+        if self._cache is not None:
+            return len(self._cache)
+        return self.execute().fetchone()[0]
+
+    def clone(self):
+        c = smartsql.Result.clone(self)
+        c._cache = None
+        return c
 
     def using(self, alias=None):
         if alias is None:
             return self._using
-        self = self.clone()
         self._using = alias
-        return self
+        self.set_compiler()
+        return self._query
 
-    def dialect(self):
-        engine = connections.databases[self.using()]['ENGINE'].rsplit('.')[-1]
-        return SMARTSQL_DIALECTS[engine]
+    def set_compiler(self):
+        engine = connections.databases[self._using]['ENGINE'].rsplit('.')[-1]
+        self.compile = SMARTSQL_COMPILERS[engine]
 
-    def sqlrepr(self, expr=None):
-        return smartsql.sqlrepr(expr or self, self.dialect())
+    def fill_cache(self):
+        if self._cache is None:
+            self._cache = list(self.iterator())
 
-    def sqlparams(self, expr=None):
-        return smartsql.sqlparams(expr or self)
-
-    def execute(self):
-        """Implementation of query execution"""
-        # TODO: sql = self._build_sql(), sqlrepr(sql, dialect), sqlparams(sql)???
-        if self._action == "select":
-            return self.model.objects.raw(self.sqlrepr(), self.sqlparams()).using(self.using())
-        return self._execute(self.sqlrepr(), self.sqlparams())
-
-    def _execute(self, sql, params):
-        cursor = connections[self.using()].cursor()
-        cursor.execute(sql, params)
-        return cursor
-
-    def result(self):
-        """Result"""
-        if self._action == 'select':
-            return self
-        if self._action == 'count':
-            return self.execute().fetchone()[0]
-        return self.execute()
-
-
-@cr
-class UnionQuerySet(smartsql.UnionQuerySet, QS):
-    """Union query class"""
-    def __init__(self, qs):
-        super(UnionQuerySet, self).__init__(qs)
-        self.model = qs.model
-        self._using = qs.using()
+    def iterator(self):
+        return self._model.objects.raw(*self.compile(self._query)).using(self._using)
 
 
 @cr
 class Table(smartsql.Table):
     """Table class for Django model"""
 
-    def __init__(self, model, qs=None, *args, **kwargs):
-        """Constructor"""
+    def __init__(self, model, q=None, *args, **kwargs):
         super(Table, self).__init__(model._meta.db_table, *args, **kwargs)
-        self.model = model
-        self._qs = qs
+        self._model = model
+        self._q = q
 
-    def _get_qs(self):
-        if isinstance(self._qs, collections.Callable):
-            self._qs = self._qs(self)
-        elif self._qs is None:
-            self._qs = QS(self).fields(self.get_fields())
-        return self._qs.clone()
+    def _get_q(self):
+        if isinstance(self._q, collections.Callable):
+            self._q = self._q(self)
+        elif self._q is None:
+            self._q = smartsql.Q(self, result=Result(self._model)).fields(self.get_fields())
+        return self._q.clone()
 
-    def _set_qs(self, val):
-        self._qs = val
+    def _set_q(self, val):
+        self._q = val
 
-    qs = property(_get_qs, _set_qs)
+    qs = property(_get_q, _set_q)
+    q = property(_get_q, _set_q)
 
     def get_fields(self, prefix=None):
-        """Returns field list."""
         if prefix is None:
             prefix = self
         result = []
-        for f in self.model._meta.local_fields:
+        for f in self._model._meta.local_fields:
             if f.column:
                 result.append(smartsql.Field(f.column, prefix))
         return result
 
-    def __getattr__(self, name):
-        """Added some django specific functional."""
-        m = self.model
-        if name[0] == '_':
-            raise AttributeError
+    def get_field(self, name):
+        m = self._model
         parts = name.split(smartsql.LOOKUP_SEP, 1)
 
         # Why do not to use responses, what returned by Signal.send()?
@@ -205,15 +169,14 @@ class Table(smartsql.Table):
         elif parts[0] in m._meta.get_all_field_names():
             parts[0] = m._meta.get_field(parts[0]).column
 
-        return super(Table, self).__getattr__(smartsql.LOOKUP_SEP.join(parts))
+        return super(Table, self).get_field(smartsql.LOOKUP_SEP.join(parts))
 
 
 @cr
 class TableAlias(smartsql.TableAlias, Table):
-    """Table alias class"""
     @property
-    def model(self):
-        return getattr(self._table, 'model', None)  # Can be subquery
+    def _model(self):
+        return getattr(self._table, '_model', None)  # Can be subquery
 
 
 @classproperty
